from __future__ import with_statement
import sqlite3


# TODO add documentation

class gpsim_database:
    """GPSIM database
    """

    __GPSIM_TABLES = [
            ("CREATE TABLE genes (gene_id INTEGER PRIMARY KEY, probe_name VARCHAR,"
             "CONSTRAINT genes_u UNIQUE (probe_name) ON CONFLICT IGNORE)"),

            ("CREATE TABLE gene_aliases (gene_id INTEGER, alias_id INTEGER, alias VARCHAR,"
             "CONSTRAINT gene_aliases_u UNIQUE (gene_id, alias_id, alias) ON CONFLICT IGNORE,"
             "CONSTRAINT gene_aliases_g_f FOREIGN KEY (gene_id) REFERENCES genes(gene_id),"
             "CONSTRAINT gene_aliases_a_f FOREIGN KEY (alias_id) REFERENCES alias_annotation(alias_id))"),

            ("CREATE TABLE z_scores (gene_id INTEGER, dataset_id INTEGER, mean_z_score DOUBLE,"
             "CONSTRAINT z_scores_u UNIQUE (gene_id, dataset_id) ON CONFLICT IGNORE,"
             "CONSTRAINT z_scores_g_f FOREIGN KEY (gene_id) REFERENCES genes(gene_id),"
             "CONSTRAINT z_scores_g_d FOREIGN KEY (dataset_id) REFERENCES expression_dataset_annotation(dataset_id))"),

            ("CREATE TABLE results (gene_id INTEGER, experiment_id INTEGER, log_likelihood DOUBLE, baseline_log_likelihood DOUBLE, params BLOB,"
             "CONSTRAINT results_u UNIQUE (gene_id, experiment_id) ON CONFLICT IGNORE,"
             "CONSTRAINT results_g_f FOREIGN KEY (gene_id) REFERENCES genes(gene_id),"
             "CONSTRAINT results_e_f FOREIGN KEY (experiment_id) REFERENCES experiment_annotation(experiment_id))"),

            ("CREATE TABLE supplementary_data (gene_id INTEGER, supp_dataset_id INTEGER, value DOUBLE,"
             "CONSTRAINT supplementary_data_u UNIQUE (gene_id, supp_dataset_id) ON CONFLICT IGNORE,"
             "CONSTRAINT supplementary_data_g_f FOREIGN KEY (gene_id) REFERENCES genes(gene_id),"
             "CONSTRAINT supplementary_data_s_f FOREIGN KEY (supp_dataset_id) REFERENCES supplementary_dataset_annotation(supp_dataset_id))"),

            ("CREATE TABLE target_sets (experiment_id INTEGER, gene_id INTEGER,"
             "CONSTRAINT target_sets_u UNIQUE (experiment_id, gene_id) ON CONFLICT IGNORE,"
             "CONSTRAINT target_sets_e_f FOREIGN KEY (experiment_id) REFERENCES experiment_annotation(experiment_id),"
             "CONSTRAINT target_sets_g_f FOREIGN KEY (gene_id) REFERENCES genes(gene_id))"),

            ("CREATE TABLE figures (experiment_id INTEGER, figure_id INTEGER,"
             "CONSTRAINT figures_u UNIQUE (experiment_id, figure_id) ON CONFLICT IGNORE,"
             "CONSTRAINT figures_e_f FOREIGN KEY (experiment_id) REFERENCES experiment_annotation(experiment_id),"
             "CONSTRAINT figures_f_f FOREIGN KEY (figure_id) REFERENCES figure_annotation(figure_id))"),

            ("CREATE TABLE figuredata (figure_id INTEGER, gene_id INTEGER, data BLOB,"
             "CONSTRAINT figuredata_u UNIQUE (figure_id, gene_id) ON CONFLICT IGNORE,"
             "CONSTRAINT figuredata_f_f FOREIGN KEY (figure_id) REFERENCES figure_annotation(figure_id),"
             "CONSTRAINT figures_g_f FOREIGN KEY (gene_id) REFERENCES genes(gene_id))"),

            ("CREATE TABLE regulators (regulator_id INTEGER PRIMARY KEY, gene_id INTEGER, dataset_id INTEGER, regulator_name VARCHAR,"
             "CONSTRAINT regulators_u UNIQUE (dataset_id, regulator_name) ON CONFLICT IGNORE,"
             "CONSTRAINT regulators_g_f FOREIGN KEY (gene_id) REFERENCES genes(gene_id),"
             "CONSTRAINT regulators_d_f FOREIGN KEY (dataset_id) REFERENCES expression_dataset_annotation(dataset_id))"),

            ("CREATE TABLE figure_annotation (figure_id INTEGER PRIMARY KEY, filename, name VARCHAR, description VARCHAR, priority INTEGER,"
             "CONSTRAINT figure_annotation_u UNIQUE (filename, name))"),

            ("CREATE TABLE experiment_annotation (experiment_id INTEGER PRIMARY KEY, dataset_id INTEGER, regulator_id INTEGER, loop_variable INTEGER, model_translation BOOLEAN, number_of_parameters INTEGER, parameter_names VARCHAR, producer VARCHAR, timestamp VARCHAR, description VARCHAR, name VARCHAR,"
             "CONSTRAINT experiment_annotation_u UNIQUE (dataset_id, regulator_id, loop_variable, model_translation, name) ON CONFLICT IGNORE,"
             "CONSTRAINT experiment_annotation_d_f FOREIGN KEY (dataset_id) REFERENCES expression_dataset_annotation(dataset_id),"
             "CONSTRAINT experiment_annotation_r_f FOREIGN KEY (regulator_id) REFERENCES regulators(regulator_id))"),

            ("CREATE TABLE experiment_set (experiment_set_id INTEGER PRIMARY KEY, parent_id INTEGER, name VARCHAR,"
             "CONSTRAINT experiment_set_u UNIQUE (name) ON CONFLICT IGNORE,"
             "CONSTRAINT experiment_set_p_f FOREIGN KEY (parent_id) REFERENCES experiment_set(experiment_set_id))"),

            ("CREATE TABLE experiment_set_experiments (experiment_set_id INTEGER, experiment_id INTEGER,"
             "CONSTRAINT experiment_set_experiments_u UNIQUE (experiment_set_id, experiment_id) ON CONFLICT IGNORE,"
             "CONSTRAINT experiment_set_experiments_s_f FOREIGN KEY (experiment_set_id) REFERENCES experiment_set(experiment_set_id),"
             "CONSTRAINT experiment_set_experiments_e_f FOREIGN KEY (experiment_id) REFERENCES experiment_annotation(experiment_id))"),

            ("CREATE TABLE expression_dataset_annotation (dataset_id INTEGER PRIMARY KEY, dataset_name VARCHAR, species VARCHAR, source VARCHAR, platform VARCHAR, description VARCHAR, save_location VARCHAR, figure_filename VARCHAR,"
             "CONSTRAINT expression_dataset_annotation_u UNIQUE (dataset_name) ON CONFLICT IGNORE)"),

            ("CREATE TABLE alias_annotation (alias_id INTEGER PRIMARY KEY, dataset_id INTEGER, alias_class VARCHAR, source VARCHAR, description VARCHAR,"
             "CONSTRAINT alias_annotation_u UNIQUE (dataset_id, alias_class) ON CONFLICT IGNORE,"
             "CONSTRAINT alias_annotation_d_f FOREIGN KEY (dataset_id) REFERENCES expression_dataset_annotation(dataset_id))"),

            ("CREATE TABLE supplementary_dataset_annotation (supp_dataset_id INTEGER PRIMARY KEY, supp_dataset_type INTEGER, supp_dataset_name VARCHAR, regulator_id INTEGER, source VARCHAR, platform VARCHAR, description VARCHAR,"
             "CONSTRAINT supplementary_dataset_annotation_u UNIQUE (supp_dataset_name, regulator_id) ON CONFLICT IGNORE,"
             "CONSTRAINT supplementary_dataset_annotation_r_f FOREIGN KEY (regulator_id) REFERENCES regulators(regulator_id))")]

    __VERSION = 1 #: Database schema version

    def __init__(self, sqlite_file):
        """Connects to a given database file.
        Creates a new file and database if it doesn't exist.
        """
        self.conn = sqlite3.connect(sqlite_file)
        self.c = self.conn.cursor()

    def __del__(self):
        """Closes the database connection and commits any transactions.
        """
        self.conn.commit()
        self.c.close()
        self.conn.close()

    def __to_placeholders(self, values):
        return ''.join(['null,' if x == None else '?,' for x in values]).strip(',')

    def __filter_nones(self, values):
        return [x for x in values if x != None]

    def __insert_table_values(self, table, values):
        self.c.execute('INSERT INTO %s VALUES (%s)'
                % (table, self.__to_placeholders(values)), self.__filter_nones(values))
        return self.c.lastrowid

    def __add_gene_aliases(self, gene_id=0, alias_id=0, alias=""):
        self.__insert_table_values("gene_aliases", (gene_id, alias_id, alias))

    def __add_z_scores(self, gene_id=0, dataset_id=0, mean_z_score=0.0):
        self.__insert_table_values("z_scores", (gene_id, dataset_id, mean_z_score))

    def __add_results(self, gene_dataset_id=0, experiment_id=0, log_likelihood=0.0, baseline_log_likelihood=None, params=None):
        self.__insert_table_values("results", (gene_dataset_id, experiment_id, log_likelihood, baseline_log_likelihood, params))

    def __add_target_sets(self, experiment_id=0, gene_id=0):
        self.__insert_table_values("target_sets", (experiment_id, gene_id))

    def __add_supplementary_data(self, gene_id=0, supp_dataset_id=0, value=0.0):
        self.__insert_table_values("supplementary_data", (gene_id, supp_dataset_id, value))

    def __add_figures(self, experiment_id=0, figure_id=0):
        self.__insert_table_values("figures", (experiment_id, figure_id))

    def __add_experiment_set_experiments(self, experiment_set_id=0, experiment_id=0):
        self.__insert_table_values("experiment_set_experiments", (experiment_set_id, experiment_id))

    def __add_experiment_set(self, parent_id=None, name=""):
        return self.__insert_table_values("experiment_set", (None, parent_id, name))

    def __add_regulators(self, gene_id=None, dataset_id=None, regulator_name=""):
        return self.__insert_table_values("regulators", (None, gene_id, dataset_id, regulator_name))

    def __add_genes(self, probe_name=""):
        return self.__insert_table_values("genes", (None, probe_name))

    def __add_figure_annotation(self, filename="", name="", description="", priority=0):
        return self.__insert_table_values("figure_annotation", (None, filename, name, description, priority))

    def __add_experiment_annotation(self, dataset_id=0, regulator_id=0,
            loop_variable=1, model_translation=False, number_of_parameters=0,
            parameter_names="", producer="", timestamp="", description="", name=""):
        return self.__insert_table_values("experiment_annotation",
                (None, dataset_id, regulator_id, loop_variable, model_translation,
                 number_of_parameters, parameter_names, producer, timestamp, description, name))

    def __add_expression_dataset_annotation(self, dataset_name="", species="",
            source="", platform="", description="", save_location="", figure_filename=""):
        return self.__insert_table_values("expression_dataset_annotation",
                (None, dataset_name, species, source, platform, description, save_location, figure_filename))

    def __add_alias_annotation(self, dataset_id=0, alias_class="", source="", description=""):
        return self.__insert_table_values("alias_annotation",
                (None, dataset_id, alias_class, source, description))

    def __add_supplementary_dataset_annotation(self, supp_dataset_type=0,
            supp_dataset_name="", regulator_id=None, source="", platform="", description=""):
        return self.__insert_table_values("supplementary_dataset_annotation",
                (None, supp_dataset_type, supp_dataset_name, regulator_id, source, platform, description))

    def __get_genes(self):
        self.c.execute("""SELECT g.probe_name, g.gene_id FROM genes AS g""")
        return self.c.fetchall()

    def __get_regulator_gene(self, regulator_name):
        self.c.execute("""SELECT ga.gene_id, a.dataset_id
                          FROM gene_aliases AS ga, alias_annotation AS a
                          WHERE ga.alias = ?
                            AND ga.alias_id = a.alias_id""", (regulator_name,))
        result = self.c.fetchone()
        if result:
            return result
        return None, None

    def __get_supplementary_annotation_id(self, name, regulator_id):
        if regulator_id:
            q = """SELECT s.supp_dataset_id
                   FROM supplementary_dataset_annotation AS s
                   WHERE s.supp_dataset_name = ?
                     AND s.regulator_id = ?"""
            a = (name, regulator_id)
        else:
            q = """SELECT s.supp_dataset_id
                   FROM supplementary_dataset_annotation AS s
                   WHERE s.supp_dataset_name = ?"""
            a = (name,)
        self.c.execute(q, a)
        result = self.c.fetchone()
        if result:
            return result[0]

    def __get_experiment_annotation_id(self, dataset_id, reg_id, name):
        self.c.execute("""SELECT e.experiment_id
                          FROM experiment_annotation AS e
                          WHERE e.dataset_id = ?
                            AND (e.regulator_id = ?
                              OR (e.regulator_id IS NULL AND e.loop_variable = 1))
                            AND e.name = ?""", (dataset_id, reg_id, name))
        result = self.c.fetchone()
        if result:
            return result[0]

    def __get_figure_annotation_id(self, filename):
        self.c.execute("""SELECT f.figure_id
                          FROM figure_annotation AS f
                          WHERE f.filename = ?""", (filename,))
        result = self.c.fetchone()
        if result:
            return result[0]

    def __get_regulator_id(self, regulator_name):
        self.c.execute("""SELECT r.regulator_id
                          FROM regulators AS r
                          WHERE r.regulator_name = ?""", (regulator_name,))
        result = self.c.fetchone()
        if result:
            return result[0]

    def __get_experiment_set_id(self, name):
        self.c.execute("""SELECT s.experiment_set_id
                          FROM experiment_set AS s
                          WHERE s.name = ?""", (name,))
        result = self.c.fetchone()
        if result:
            return result[0]

    def __get_expression_dataset_annotation_id(self, name):
        self.c.execute("""SELECT e.dataset_id
                          FROM expression_dataset_annotation AS e
                          WHERE e.dataset_name = ?""", (name,))
        result = self.c.fetchone()
        if result:
            return result[0]

    def __get_alias_annotation_id(self, dataset_id, alias_class, source, desc):
        self.c.execute("""SELECT a.alias_id
                          FROM alias_annotation AS a
                          WHERE a.dataset_id = ?
                            AND a.alias_class = ?""", (dataset_id, alias_class))
        result = self.c.fetchone()
        if result:
            return result[0]

    def __get_genes_with_supplementary_data(self, supp_dataset_id):
        self.c.execute("""SELECT s.gene_id
                          FROM supplementary_data AS s
                          WHERE s.supp_dataset_id = ?""", (supp_dataset_id,))
        return [gene_id for (gene_id,) in self.c.fetchall()]

    def __get_genes_with_zscore(self, dataset_id):
        self.c.execute("""SELECT z.gene_id
                          FROM z_scores AS z
                          WHERE z.dataset_id = ?""", (dataset_id,))
        return [gene_id for (gene_id,) in self.c.fetchall()]

    def __get_genes_with_result(self, experiment_id):
        self.c.execute("""SELECT r.gene_id
                          FROM results AS r
                          WHERE r.experiment_id = ?""", (experiment_id,))
        return [gene_id for (gene_id,) in self.c.fetchall()]

    def __get_genes_with_aliases(self, alias_id):
        self.c.execute("""SELECT a.gene_id
                          FROM gene_aliases AS a
                          WHERE a.alias_id = ?""", (alias_id,))
        return [gene_id for (gene_id,) in self.c.fetchall()]

    def __get_genes_with_experiment_target(self, experiment_id):
        self.c.execute("""SELECT t.gene_id
                          FROM target_sets AS t
                          WHERE t.experiment_id = ?""", (experiment_id,))
        return [gene_id for (gene_id,) in self.c.fetchall()]

    def __get_experiments_in_set(self, set_id):
        self.c.execute("""SELECT s.experiment_id
                          FROM experiment_set_experiments AS s
                          WHERE s.experiment_set_id = ?""", (set_id,))
        return [exp_id for (exp_id,) in self.c.fetchall()]

    def __get_experiment_figures(self, experiment_id):
        self.c.execute("""SELECT f.filename
                          FROM figure_annotation AS f, figures AS fi
                          WHERE fi.experiment_id = ?
                            AND fi.figure_id = f.figure_id""", (experiment_id,))
        return [filename for (filename,) in self.c.fetchall()]


    def create_tables(self):
        """Tries to create tables for the database.
        Throws an exception if a table cannot be created.
        """
        for l in self.__GPSIM_TABLES:
            self.c.execute(l)
        self.conn.commit()
        self.c.execute('PRAGMA user_version = ' + str(self.__VERSION))
        self.c.execute('CREATE INDEX results_index ON results (gene_id)')

    def get_version(self):
        """Gets the database schema version.
        """
        self.c.execute('PRAGMA user_version')
        return self.c.fetchone()[0]

    def dump_to_file(self, filename):
        """Dumps the SQL command log of this database connection.
        """
        self.conn.commit()
        with open(filename, 'w') as f:
            for line in self.conn.iterdump():
                f.write('%s\n' % line)

    def add_and_get_regulator_id(self, regulator_name):
        """Gets the regulator id.
        Adds the regulator to the database if necessary.
        """
        if not regulator_name:
            return None
        reg_id = self.__get_regulator_id(regulator_name)
        if not reg_id:
            gene_id, dataset_id = self.__get_regulator_gene(regulator_name)
            reg_id = self.__add_regulators(gene_id, dataset_id, regulator_name)
        return reg_id

    def add_and_get_genes_dict(self, probe_names):
        """Gets a dictionary of gene probe names and ids.
        For example: {'FBgn0000008': 1,'FBgn0000011': 2,'FBgn0000014': 3}
        Adds missing genes to the database.
        """
        names = []
        ids = []
        genes = self.__get_genes()
        if genes:
            names, ids = list(zip(*genes))
        missing = set(probe_names) - set(names)
        for name in missing:
            genes.append((name, self.__add_genes(name)))
        dict = {}
        for (name, id) in genes:
            dict[name] = id
        return dict

    def add_and_get_experiment_set(self, set_name, parent_name):
        """Gets the experiment set id.
        Adds a new experiment set if necessary.
        """
        if not set_name:
            return None
        exp_set_id = self.__get_experiment_set_id(set_name)
        if not exp_set_id:
            exp_set_id = self.__add_experiment_set(self.add_and_get_experiment_set(parent_name, None), set_name)
        return exp_set_id

    def add_and_get_supplementary_annotation(self, type, name, regulator_name, source, platform, desc):
        """Gets the supplementary annotation id.
        Adds the annotation to the database if necessary.
        Adds the regulator to the database if necessary.
        """
        reg_id = self.add_and_get_regulator_id(regulator_name)
        supp_id = self.__get_supplementary_annotation_id(name, reg_id)
        if not supp_id:
            supp_id = self.__add_supplementary_dataset_annotation(type, name, reg_id, source, platform, desc)
        return supp_id

    def add_and_get_dataset_annotation(self, name, species, source, platform, desc, save_location, figure_filename):
        """Gets the dataset annotation id.
        Adds the annotation to the database if necessary.
        """
        dataset_id = self.__get_expression_dataset_annotation_id(name)
        if not dataset_id:
            dataset_id = self.__add_expression_dataset_annotation(name, species, source, platform, desc, save_location, figure_filename)
        return dataset_id

    def add_and_get_experiment_annotation(self, dataset_id, regulator_name, loop_variable, model_translation, number_of_params, param_names, producer, timestamp, desc, name):
        """Gets the experiment annotation id.
        Adds the annotation to the database if necessary.
        Adds the regulator to the database if necessary.
        """
        reg_id = self.add_and_get_regulator_id(regulator_name)
        exp_id = self.__get_experiment_annotation_id(dataset_id, reg_id, name)
        if not exp_id:
            exp_id = self.__add_experiment_annotation(dataset_id, reg_id, loop_variable, model_translation, number_of_params, param_names, producer, timestamp, desc, name)
        return exp_id

    def add_and_get_alias_annotation(self, dataset_id, alias_class, source, description):
        """Gets the alias annotation id.
        Adds the annotation to the database if necessary.
        """
        alias_id = self.__get_alias_annotation_id(dataset_id, alias_class, source, description)
        if not alias_id:
            alias_id = self.__add_alias_annotation(dataset_id, alias_class, source, description)
        return alias_id

    def add_and_get_figure_annotation(self, filename, name, description, priority):
        """Gets the figure annotation id.
        Adds the annotation to the database if necessary.
        """
        figure_id = self.__get_figure_annotation_id(filename)
        if not figure_id:
            figure_id = self.__add_figure_annotation(filename, name, description, priority)
        return figure_id
     
    def add_supplementary_data(self, probe_names, values, supp_annotation):
        """Adds the given supplementary data to the given probe_names.
        Adds missing genes to the database.
        Adds the supplementary annotation if it doesn't exist.
        Adds the regulator if necessary.

        supp_annotation = [type, name, regulator_name, source, platform, description]
        """
        genes_dict = self.add_and_get_genes_dict(probe_names)
        gene_ids = list(genes_dict.values())
        supp_id = self.add_and_get_supplementary_annotation(*supp_annotation)
        missing = set(gene_ids) - set(self.__get_genes_with_supplementary_data(supp_id))
        for (probe_name, value) in zip(probe_names, values):
            gene_id = genes_dict[probe_name]
            if gene_id in missing:
                self.__add_supplementary_data(gene_id, supp_id, value)

    def add_zscores(self, probe_names, zscores, dataset_annotation):
        """Adds zscores to the given genes.
        Adds missing genes to the database.
        Adds dataset_annotation if it doesn't exist yet.

        dataset_annotation = [name, species, source, platform, description, save_locatio, figure_filename]
        """
        genes_dict = self.add_and_get_genes_dict(probe_names)
        gene_ids = list(genes_dict.values())
        dataset_id = self.add_and_get_dataset_annotation(*dataset_annotation)
        missing = set(gene_ids) - set(self.__get_genes_with_zscore(dataset_id))
        for (probe_name, value) in zip(probe_names, zscores):
            gene_id = genes_dict[probe_name]
            if gene_id in missing:
                self.__add_z_scores(gene_id, dataset_id, value)

    def add_experiment_results(self, probe_names, log_likelihoods, baseline_log_likelihoods, params, experiment_annotation, dataset_annotation):
        """Adds experiment results to the given genes.
        Adds missing genes to the database.
        Adds the experiment anntation if it doesn't exist.
        Adds dataset annotation if necessary.

        experiment_annotation = [regulator_name, loop_variable, model_translation, number_of_params, param_names, producer, timestamp, desc, name]
        dataset_annotation = [name, species, source, platform, description, save_locatio, figure_filename]
        """
        genes_dict = self.add_and_get_genes_dict(probe_names)
        gene_ids = list(genes_dict.values())
        dataset_id = self.add_and_get_dataset_annotation(*dataset_annotation)
        experiment_id = self.add_and_get_experiment_annotation(*([dataset_id] + experiment_annotation))
        missing = set(gene_ids) - set(self.__get_genes_with_result(experiment_id))
        for (probe_name, log_likelihood, baseline_log_likelihood, param) in zip(probe_names, log_likelihoods, baseline_log_likelihoods, params):
            gene_id = genes_dict[probe_name]
            if gene_id in missing:
                self.__add_results(gene_id, experiment_id, log_likelihood, baseline_log_likelihood, param)

    def add_experiment_set(self, set_name, parent_name, experiment_annotations, dataset_annotations):
        """Adds the given experiment to the given experiment set.
        Adds missing experiment sets (set and its parent) if necessary.
        Adds missing experiment annotations if necessary.
        Adds missing dataseset annotations if necessary.

        experiment_annotations = [[regulator_name, loop_variable, model_translation, number_of_params, param_names, producer, timestamp, desc, name]]
        dataset_annotations = [[name, species, source, platform, description, save_locatio, figure_filename]]
        """
        set_id = self.add_and_get_experiment_set(set_name, parent_name)
        exp_ids = []
        for (exp_ann, data_ann) in zip(experiment_annotations, dataset_annotations):
            dataset_id = self.add_and_get_dataset_annotation(*data_ann)
            exp_ids.append(self.add_and_get_experiment_annotation(*([dataset_id] + exp_ann)))
        missing = set(exp_ids) - set(self.__get_experiments_in_set(set_id))
        for exp_id in missing:
            self.__add_experiment_set_experiments(set_id, exp_id)

    def add_figure(self, experiment_annotation, dataset_annotation, figure_annotation):
        """Adds the given figure to the given experiment.
        Adds missing experiment annotation if necessary.
        Adds missing dataset annotation if necessary.
        Adds missing figure annotation if necessary.

        experiment_annotation = [regulator_name, loop_variable, model_translation, number_of_params, param_names, producer, timestamp, desc, name]
        dataset_annotation = [name, species, source, platform, description, save_locatio, figure_filename]
        figure_annotation = [filename, name, description, priority]
        """
        dataset_id = self.add_and_get_dataset_annotation(*dataset_annotation)
        experiment_id = self.add_and_get_experiment_annotation(*([dataset_id] + experiment_annotation))
        figure_id = self.add_and_get_figure_annotation(*figure_annotation)
        if figure_id not in self.__get_experiment_figures(experiment_id):
            self.__add_figures(experiment_id, figure_id)

    def add_target_sets(self, probe_names, experiment_annotation, dataset_annotation):
        """Adds given genes to the target set of a given experiment.
        Adds missing experiment annotation if necessary.
        Adds missing genes if necessary.
        """
        genes_dict = self.add_and_get_genes_dict(probe_names)
        gene_ids = list(genes_dict.values())
        dataset_id = self.add_and_get_dataset_annotation(*dataset_annotation)
        experiment_id = self.add_and_get_experiment_annotation(*([dataset_id] + experiment_annotation))
        missing = set(gene_ids) - set(self.__get_genes_with_experiment_target(experiment_id))
        for probe_name in probe_names:
            gene_id = genes_dict[probe_name]
            if gene_id in missing:
                self.__add_target_sets(experiment_id, gene_id)

    def add_aliases(self, probe_names, aliases, alias_annotation, dataset_annotation):
        """Adds aliases to genes.
        Adds missing genes.
        Adds missing alias annotation.
        Adds missing dataset annotation.

        dataset_annotations = [name, species, source, platform, description, save_locatio, figure_filename]
        alias_annotation = [alias_class, source, description]
        """
        genes_dict = self.add_and_get_genes_dict(probe_names)
        gene_ids = list(genes_dict.values())
        dataset_id = self.add_and_get_dataset_annotation(*dataset_annotation)
        alias_id = self.add_and_get_alias_annotation(*([dataset_id] + alias_annotation))
        missing = set(gene_ids) - set(self.__get_genes_with_aliases(alias_id))
        for (probe_name, alias) in zip(probe_names, aliases):
            gene_id = genes_dict[probe_name]
            if gene_id in missing:
                self.__add_gene_aliases(gene_id, alias_id, alias)

    def add_regulator(self, regulator_name, probe_name, dataset_annotation):
        """Adds the given regulator to the database.
        Adds dataset annotation if necessary.
        Adds a gene if probe_name is defined.
        """
        reg_id = self.__get_regulator_id(regulator_name)
        if reg_id:
            return
        gene_id = None
        if probe_name:
            genes_dict = self.add_and_get_genes_dict([probe_name])
            gene_id = genes_dict[probe_name]
        
        dataset_id = self.add_and_get_dataset_annotation(*dataset_annotation)
        self.__add_regulators(gene_id, dataset_id, regulator_name)




#!/usr/bin/python
import sys
import os
import webbrowser
from optparse import OptionParser, OptionGroup
from threading import Thread
import pkg_resources

try: # Python >=3.0
    from configparser import RawConfigParser
    from http.server import CGIHTTPRequestHandler, HTTPServer
    from tkinter import *
    from tkinter.filedialog import askopenfilename
    from tkinter.messagebox import *
except ImportError: # Python <3.0
    from ConfigParser import RawConfigParser
    from CGIHTTPServer import CGIHTTPRequestHandler
    from BaseHTTPServer import HTTPServer
    from Tkinter import *
    from tkFileDialog import askopenfilename
    from tkMessageBox import *

# Config file for server
CONFIG_FILE = 'tigreBrowser.cfg'
DEFAULT_CONFIG_FILE = pkg_resources.resource_filename('tigreBrowser', 'cgi/tigreBrowser/tigreBrowser.cfg')
CGI_DIR = pkg_resources.resource_filename('tigreBrowser', 'cgi')
BROWSER_DIR = '/tigreBrowser'

def start_server(port):
    """Starts a CGI HTTP server at the given port.
    CGI scripts (.cgi file extension) are executed in the BROWSER_DIR
    directory.
    """
    class Handler(CGIHTTPRequestHandler):
        cgi_directories = [BROWSER_DIR]
        def is_cgi(self):
            """The pathname is considered a CGI file if the CGIHTTPRequestHandler
            returns true for is_cgi(), the file is executable and the file
            extension of the file is .cgi.
            """
            path = self.path
            match = False

            for x in self.cgi_directories:
                i = len(x)
                if path[:i] == x and (not path[i:] or path[i] == '/'):
                    self.cgi_info = path[:i], path[i+1:]
                    match = True

            if match:
                name = self.path.split('?', 2)[0]
                file = self.translate_path(name)
                if not self.is_executable(file):
                    return False
                if not file.endswith('.cgi'):
                    return False
                return True
            return False

    os.chdir(CGI_DIR)
    httpd = HTTPServer(("", port), Handler)
    print("Result browser running on http://localhost:" + str(port) + "/" + BROWSER_DIR.strip('/') + "/tigreBrowser.cgi")

    try:
        httpd.serve_forever()
    except (KeyboardInterrupt, SystemExit):
        print("\nTerminating result browser")

def read_config_file(config_file):
    """Reads the config file.
    Returns port setting from the config file.
    """
    config = RawConfigParser()
    if not config.read(config_file):
        print("Could not find config file '%s'" % config_file)
        return 9999

    try:
        port = config.getint('server', 'port')
    except (Exception,):
        e = sys.exc_info()[1]
        print("Error in config file: %s" % e)
        sys.exit(1)
    return port

def read_commandline_options(config_file):
    """Reads command-line options.
    If database file is given as an option, the gui doesn't have to run.
    Also, this function passes the database file as an environment variable
    to the actual browser.
    """
    usage = "These options override the options set in the '%s' config file\n" % config_file + "usage: %prog [options]"
    parser = OptionParser(usage)
    group_server = OptionGroup(parser, 'Server options')
    group_browser = OptionGroup(parser, 'Result browser options')

    group_server.add_option('-p', '--port', dest='port', type='int',
                            action='store', help='server port', default=None)

    group_browser.add_option('-d', '--database', dest='database_file',
                             type='string', action='store',
                             help='database file for the result browser',
                             default=None)

    parser.add_option_group(group_server)
    parser.add_option_group(group_browser)
    (options, args) = parser.parse_args()

    run_gui = True
    if options.database_file:
        os.environ['RESULT_BROWSER_DATABASE'] = os.path.join(os.getcwd(), options.database_file)
        run_gui = False

    return options.port, run_gui

def get_database_file_error(database_file):
    """Check if the database file is OK for the browser (i.e. the file exists
    and the file is not writable)

    Returns an error string describing the issue with the database file
    or None if there are no issues.
    """
    if not os.path.isfile(database_file):
        return "Database file not found"
    if os.access(database_file, os.W_OK):
        return "Database file must not have write permissions"

class TigreBrowser(Frame):
    """GUI config window for tigreServer and tigreBrowser.
    Creates a window with database file selection and controls to start
    or stop the server.
    """
    def __init__(self, port, master=None):
        """Creates a new window.
        """
        Frame.__init__(self, master)
        self.root = master
        self.root.protocol("WM_DELETE_WINDOW", sys.exit)
        self.thread = None
        self.port = port
        self.pack()
        self.__create_widgets()

    def __create_widgets(self):
        """Creates all widgets.
        """
        self.title = Label(self, text="tigreBrowser", font="Helvetica 20", anchor=NW, justify=LEFT, pady=10)
        self.title.pack({"side": "top"})

        self.file_frame = Frame(self, pady=10)
        self.file_frame.pack()

        self.label = Label(self.file_frame, text="Select a database file")
        self.label.pack({"side": "left"})

        self.open = Button(self.file_frame, text="Open database file", command=self.__open_db)
        self.open.pack({"side": "right"})

        self.button_frame = Frame(self)
        self.button_frame.pack()

        self.start = Button(self.button_frame, text="Start server", command=self.__start_server, state=DISABLED)
        self.start.pack({"side": "left"})

        self.stop = Button(self.button_frame, text="Stop server", command=self.__stop_server, state=DISABLED)
        self.stop.pack({"side": "right"})

    def __open_db(self):
        """Opens a file dialog for user to select the database file.
        Shows an error in case on an error.
        Adds a label indicating the filename after the selection is done.
        Also, passes the database file to tigreBrowser using an environment variable.
        """
        database_file = askopenfilename(filetypes=[("SQLite database files", "*.sqlite"), ("All files", "*")])
        if not database_file:
            return
        message = get_database_file_error(database_file)
        if message:
            showerror("Error!", message)
            return
        os.environ['RESULT_BROWSER_DATABASE'] = database_file
        self.start.config(state=NORMAL)

        self.filename = Label(self, text=database_file, pady=5)
        self.filename.pack()

    def __start_server(self):
        """Starts the CGI HTTP server as a new thread.
        Adds a label indicating the URL of tigreBrowser.
        """
        print("Starting server")
        self.thread = Thread(target=start_server, args=(self.port, ))
        self.thread.start()
        self.start.config(state=DISABLED)
        self.open.config(state=DISABLED)
        self.stop.config(state=NORMAL)
        url = "http://localhost:" + str(port) + "/" + BROWSER_DIR.strip('/') + "/tigreBrowser.cgi"
        l = Label(self, text="Result browser running on:")
        l.pack()
        button = Button(self, text=url, command=lambda: webbrowser.open(url), fg="blue", activeforeground="blue", bd=0, font=("Helvetica", 11, "underline"))
        button.pack()

    def __stop_server(self):
        """Stops the server and exits the application.
        Note: there is no way to just kill the thread running the server.
        Therefore one must exit the whole application.
        """
        print("Stopping server")
        self.start.config(state=NORMAL)
        self.open.config(state=NORMAL)
        sys.exit()

def create_gui(port):
    """Creates and starts the server GUI.
    """
    root = Tk()
    app = TigreBrowser(port, master=root)
    root.title("TigreBrowser")
    app.mainloop()
    root.destroy

if __name__ == "__main__":
    if os.path.isfile(CONFIG_FILE):
        myconfig = CONFIG_FILE
        os.environ['TIGREBROWSER_CONFIG'] = os.path.join(os.getcwd(), myconfig)
    else:
        myconfig = DEFAULT_CONFIG_FILE
    port_cfg = read_config_file(myconfig)
    port_cli, run_gui = read_commandline_options(myconfig)
    port = port_cli or port_cfg

    if run_gui:
        create_gui(port)
    else:
        start_server(port)



#! /usr/bin/env python

import sqlite3

class Database:
    """GPSIM database access for results browser.
    """

    def __init__(self, sqlite_file):
        """Connects to a given database file.
        """
        self.conn = sqlite3.connect(sqlite_file)
        self.c = self.conn.cursor()

    def __del__(self):
        """Closes the database connection and commits any transactions.
        """
        self.conn.commit()
        self.c.close()
        self.conn.close()

    def __to_placeholders(self, values):
        return ''.join(['null,' if x == None else '?,' for x in values]).strip(',')

    def __get_experiment_ids_in_sets(self, set_ids):
        self.c.execute("""SELECT DISTINCT se.experiment_id
                          FROM experiment_set_experiments AS se
                          WHERE se.experiment_set_id IN (%s)""" % self.__to_placeholders(set_ids), set_ids)
        return [id for (id,) in self.c.fetchall()]

    def __get_experiment_sets_children(self, set_ids):
        self.c.execute("""SELECT DISTINCT s.experiment_set_id
                          FROM experiment_set AS s
                          WHERE s.parent_id IN (%s)""" % self.__to_placeholders(set_ids), set_ids)
        return [id for (id,) in self.c.fetchall()]

    def __create_supplementary_data_query(self, id, symbol, value, gene_ids=None):
        if not gene_ids:
            query = """SELECT supp.gene_id
                       FROM supplementary_data AS supp
                       WHERE supp.supp_dataset_id = ?
                         AND supp.value %s ?""" % symbol
            args = [id, value]
        else:
            query = """SELECT supp.gene_id
                       FROM supplementary_data AS supp
                       WHERE supp.supp_dataset_id = ?
                         AND supp.value %s ?
                         AND supp.gene_id IN (%s)""" % (symbol, self.__to_placeholders(gene_ids))
            args = [id, value] + gene_ids

        return query, args

    def __create_supplementary_data_queries(self, supp_options, supp_ids):
        query = []
        args = []
        for (name, symbol, value) in supp_options:
            q, a = self.__create_supplementary_data_query(supp_ids[name], symbol, float(value))
            query.append(q)
            args += a
        return '\nINTERSECT\n'.join(query), args

    def __create_log_likelihood_filter(self, criteria, experiment_id, diff=False):
        query = """SELECT r.gene_id
                   FROM results AS r
                   WHERE r.experiment_id = ?"""
        args = [experiment_id]

        if diff:
            q = ' AND (r.log_likelihood - r.baseline_log_likelihood) %s ? '
        else:
            q = ' AND r.log_likelihood %s ? '

        for (symbol, value) in criteria:
            query += q % symbol
            args.append(value)

        return query, args

    def __create_baseline_filter(self, criteria, experiment_id):
        query = """SELECT DISTINCT r.gene_id
                   FROM results AS r
                   WHERE r.experiment_id = ?"""
        args = [experiment_id]

        q = ' AND r.baseline_log_likelihood %s ? '
        for (symbol, value) in criteria:
            query += q % symbol
            args.append(value)

        return query, args

    def __create_z_score_filter(self, criteria):
        query = """SELECT z.gene_id
                   FROM z_scores AS z
                   WHERE 1"""
        args = []

        q = ' AND z.mean_z_score %s ? '
        for (symbol, value) in criteria:
            query += q % symbol
            args.append(value)

        return query, args

    def __create_filter(self, filterby, criteria, experiment_id, baseline=False, diff=False):
        if baseline:
            return self.__create_baseline_filter(criteria, experiment_id)
        elif filterby == 'zscore':
            return self.__create_z_score_filter(criteria)
        else:
            return self.__create_log_likelihood_filter(criteria, experiment_id, diff)

    def get_experiment_ids_in_set_recursively(self, set_id):
        """Get all experiment ids that belong to the given experiment set or any subset of it.

        Returns: [experiment id]
        """
        ids = []
        children = [set_id]
        while len(children) > 0:
            ids.extend(self.__get_experiment_ids_in_sets(children))
            children = self.__get_experiment_sets_children(children)

        return ids

    def get_experiment_sets(self):
        """Gets all experiment sets.

        Returns: [(set name, set id, set parent id)]
        """
        self.c.execute("""SELECT s.name, s.experiment_set_id, s.parent_id
                          FROM experiment_set AS s""")
        return self.c.fetchall()

    def get_experiment_set_id(self, set_name):
        """Gets the id of the given experiment set name.
        """
        self.c.execute("""SELECT s.experiment_set_id
                          FROM experiment_set AS s
                          WHERE s.name = ?""", (set_name,))
        result = self.c.fetchone()
        if result:
            return result[0]

    def get_regulator_id(self, name):
        """Gets the regulator id matching the given regulator name.

        Returns: id
        """
        self.c.execute("""SELECT reg.regulator_id
                          FROM regulators AS reg
                          WHERE reg.regulator_name = ?""", (name,))
        result = self.c.fetchone()
        if result:
            return result[0]

    def get_experiment_id(self, name, reg_id):
        """Gets the experiment id matching the given experiment name and regulator id.
        If reg_id is None, experiment is returned if the name matches and if
        the loop_variable of the experiment is 1 and the regulator of the experiment
        is NULL.

        Returns: id
        """
        if reg_id:
            self.c.execute('SELECT e.experiment_id FROM experiment_annotation AS e WHERE e.name = ? AND e.regulator_id = ?', (name, reg_id))
        else:
            self.c.execute('SELECT e.experiment_id FROM experiment_annotation AS e WHERE e.name = ? AND e.regulator_id IS NULL AND e.loop_variable = 1', (name,))
        result = self.c.fetchone()
        if result:
            return result[0]

    def get_experiment_ids(self, reg_id):
        """Gets all experiment ids matching the given regulator id.

        Returns: [experiment id]
        """
        self.c.execute('SELECT e.experiment_id FROM experiment_annotation AS e WHERE e.regulator_id = ?', (reg_id,))
        return [id for (id,) in self.c.fetchall()]

    def get_experiment_dataset_id(self, experiment_id):
        """Gets the dataset id for the given experiment.

        Returns: id
        """
        self.c.execute("""SELECT e.dataset_id
                          FROM experiment_annotation AS e
                          WHERE e.experiment_id = ?""", (experiment_id,))
        return self.c.fetchone()[0]

    def get_regulators(self):
        """Gets all regulator names.

        Returns: [regulator name]
        """
        self.c.execute('SELECT regulators.regulator_name FROM regulators')
        return [reg for (reg,) in self.c.fetchall()]

    def get_target_sets(self):
        """Gets all target sets.

        Returns a mapping from experiment_id to target gene ids.
        For example:
        {1: [494], 2: [123, 456]}
        """
        self.c.execute("""SELECT t.experiment_id, t.gene_id
                          FROM target_sets AS t""")
        d = {}
        for (exp_id, gene_id) in self.c.fetchall():
            d[exp_id] = gene_id
        return d

    def get_supplementary_annotations(self):
        """Gets all supplementary annotations.

        Returns: [(dataset type, dataset name, description)]
        """
        self.c.execute('SELECT DISTINCT s.supp_dataset_type, s.supp_dataset_name, s.description FROM supplementary_dataset_annotation AS s')
        return self.c.fetchall()

    def get_supplementary_annotation_ids(self, reg_id):
        """Gets supplementary annotation names and ids that have either the
        given regulator or no regulator at all.

        Returns: [dataset name, dataset id]
        """
        self.c.execute("""SELECT s.supp_dataset_name, s.supp_dataset_id
                          FROM supplementary_dataset_annotation AS s
                          WHERE (s.regulator_id IS NULL OR s.regulator_id = ?)""", (reg_id,))
        return self.c.fetchall()

    def get_experiment_names(self):
        """Gets all experiment names.

        Returns: [experiment name]
        """
        self.c.execute('SELECT DISTINCT e.name FROM experiment_annotation AS e')
        return [name for (name,) in self.c.fetchall()]

    def get_z_scores(self, gene_ids, dataset_id):
        """Gets z-scores for the given genes in the given dataset.

        Returns: [gene id, zscore]
        """
        args = [dataset_id] + gene_ids
        self.c.execute("""SELECT z.gene_id, z.mean_z_score
                          FROM z_scores AS z
                          WHERE z.dataset_id = ?
                            AND z.gene_id IN (%s)""" % self.__to_placeholders(gene_ids), args)
        return self.c.fetchall()

    def get_gene_ids_by_name_in_experiments(self, experiment_ids, names):
        """Gets all gene ids that have results in the given experiments and that
        have matching aliases or probe_names.
        """
        query = """SELECT DISTINCT ga.gene_id
                   FROM gene_aliases AS ga, results AS r
                   WHERE ga.alias IN (%s)
                   AND ga.gene_id = r.gene_id
                   AND r.experiment_id IN (%s)
                   UNION
                   SELECT DISTINCT g.gene_id
                   FROM genes AS g, results AS r
                   WHERE g.probe_name IN (%s)
                   AND g.gene_id = r.gene_id
                   AND r.experiment_id IN (%s)""" % (self.__to_placeholders(names), self.__to_placeholders(experiment_ids), self.__to_placeholders(names), self.__to_placeholders(experiment_ids))
        args = names + experiment_ids + names + experiment_ids
        self.c.execute(query, args)
        return [gene_id for (gene_id,) in self.c.fetchall()]

    def get_gene_aliases(self, gene_ids):
        """Gets all aliases for the given gene ids.

        Returns: [gene id, alias class, alias]
        """
        self.c.execute("""SELECT ga.gene_id, aa.alias_class, ga.alias
                          FROM gene_aliases AS ga, alias_annotation AS aa
                          WHERE ga.alias_id = aa.alias_id
                            AND gene_id IN (%s)""" % self.__to_placeholders(gene_ids), gene_ids)
        return self.c.fetchall()

    def get_alias_annotations(self, dataset_id):
        """Gets all alias annotations for the given dataset.

        Returns: [alias id, alias class, source, description]
        """
        self.c.execute("""SELECT aa.alias_id, aa.alias_class, aa.source, aa.description
                          FROM alias_annotation AS aa
                          WHERE aa.dataset_id = ?""", (dataset_id,))
        return self.c.fetchall()

    def get_dataset_figure_filename(self, dataset_id):
        """Gets the figure filename for the given dataset.

        Returns: figure_filename
        """
        self.c.execute("""SELECT d.figure_filename
                          FROM expression_dataset_annotation AS d
                          WHERE d.dataset_id = ?""", (dataset_id,))
        filename = self.c.fetchone()
        if filename:
            return filename[0]

    def get_experiment_figures(self, experiment_ids):
        """Gets all unique figure annotations for the given experiment ids.

        Returns: [(figure id, filename, name, description, priority)]
        """
        self.c.execute("""SELECT DISTINCT fa.figure_id, fa.filename, fa.name, fa.description, fa.priority
                          FROM figures AS f, figure_annotation AS fa
                          WHERE f.figure_id = fa.figure_id
                            AND f.experiment_id IN (%s)
                            ORDER BY fa.priority DESC""" % self.__to_placeholders(experiment_ids), tuple(experiment_ids))
        return self.c.fetchall()

    def get_figure_annotations(self):
        """Gets all figure annotations.

        Returns: [(figure_id, filename, name, description, priority)]
        """
        self.c.execute("""SELECT f.figure_id, f.filename, f.name, f.description, f.priority
                          FROM figure_annotation AS f""")
        return self.c.fetchall()

    def get_figure_experiments(self, figure_id):
        """Gets all experiment ids which have the given figure.
        """
        self.c.execute("""SELECT f.experiment_id
                          FROM figures AS f
                          WHERE f.figure_id = ?""", (figure_id,))
        return [experiment_id for (experiment_id,) in self.c.fetchall()]

    def get_experiment_name(self, experiment_id):
        """Gets experiment name.

        Returns: experiment_name
        """
        self.c.execute("""SELECT e.name
                          FROM experiment_annotation AS e
                          WHERE e.experiment_id = ?""", (experiment_id,))
        result = self.c.fetchone()
        if result:
            return result[0]

    def get_experiment_regulator_name(self, experiment_id):
        """Gets the regulator in the experiment.

        Returns: regulator_name
        """
        self.c.execute("""SELECT r.regulator_name
                          FROM regulators AS r, experiment_annotation AS e
                          WHERE e.regulator_id = r.regulator_id
                            AND e.experiment_id = ?""", (experiment_id,))
        result = self.c.fetchone()
        if result:
            return result[0]

    def get_experiment_target_genes(self, experiment_id):
        """Gets target gene names of the given experiment.

        Returns: [probe_name]
        """
        self.c.execute("""SELECT g.probe_name
                          FROM genes AS g, target_sets AS t
                          WHERE t.experiment_id = ?
                            AND g.gene_id = t.gene_id""", (experiment_id,))
        return [probe_name for (probe_name,) in self.c.fetchall()]

    def get_experiment_dataset_name(self, experiment_id):
        """Gets the dataset name of the given experiment.

        Returns: dataset_name
        """
        self.c.execute("""SELECT d.dataset_name
                          FROM expression_dataset_annotation AS d, experiment_annotation AS e
                          WHERE d.dataset_id = e.dataset_id""")
        result = self.c.fetchone()
        if result:
            return result[0]

    def get_dataset_annotations(self):
        """Gets all dataset annotations.

        Returns: [(dataset_id, dataset_name, species, source, platform, desc, save_location, figure_filename)]
        """
        self.c.execute("""SELECT d.dataset_id, d.dataset_name, d.species, d.source, d.platform, d.description, d.save_location, d.figure_filename
                          FROM expression_dataset_annotation AS d""")
        return self.c.fetchall()

    def get_gene_supplementary_datas(self, gene_ids, supp_ids):
        """Gets the given supplementary data for the given gene ids.

        Returns: [gene id, dataset name, value]
        """
        self.c.execute("""SELECT supp.gene_id, supp_a.supp_dataset_name, supp.value
                          FROM supplementary_data AS supp, supplementary_dataset_annotation AS supp_a
                          WHERE supp.supp_dataset_id = supp_a.supp_dataset_id
                            AND supp.gene_id IN (%s)
                            AND supp_a.supp_dataset_id IN (%s)""" % (self.__to_placeholders(gene_ids), self.__to_placeholders(supp_ids)), gene_ids + list(supp_ids.values()))

        return self.c.fetchall()

    def create_filter_query(self, reg_id, filters, supp_options, supp_ids):
        """Creates a filter query.

        Parameters:

        filters = dictionary of the given form: {entity: [(symbol, value)]}
        The entity can be either an experiment name or 'zscore'. The experiment
        name can have '_baseline' or '_diff' appended to it in which case
        the filter is done by baseline_log_likelihood or by
        log_likelihood - baseline_log_likelihood correspondingly.
        Symbol can be one of the following: '>', '>=', '=', '<=', '<'.
        Value must be a number.

        Example:
        filters = {'GPSIM_diff': [('>', 0.0)], 'zscore': [('=', 0.0)], 'GPDISIM': [('>', 4.0), ('<', 500.0)]}

        supp_options = list of the following form: [(supplementary dataset name, symbol, value)]
        Symbol can be one of the following: '>', '>=', '=', '<=', '<'.
        Value must be a number.

        Example:
        supp_options = [(u'ischip', '=', '1.0'), (u'hasinsitu', '=', '0.0'), (u'chipdist', '>', '3')]

        supp_ids = dictionary of supplementary dataset names to their ids.
        NOTE: The regulator in the supplementary dataset must match reg_id or be NULL.

        Example:
        supp_ids = {u'ischip': 4, u'chipdist': 19, u'isinsitu': 3, u'hasinsitu': 2}


        Returns: (query string, query arguments)

        The query string is used to find all gene ids that match the given filters.
        """
        if not filters:
            return self.__create_supplementary_data_queries(supp_options, supp_ids)

        query = []
        args = []
        for (name, criteria) in list(filters.items()):
            parts = name.split('_')
            filterby = parts[0]
            baseline = False
            diff = False
            if len(parts) > 1:
                filterby = '_'.join(parts[:-1])
                if parts[-1] == 'baseline':
                    baseline = True
                elif parts[-1] == 'diff':
                    diff = True
                else:
                    filterby = name
            experiment_id = self.get_experiment_id(filterby, reg_id)
            q, a = self.__create_filter(filterby, criteria, experiment_id, baseline, diff)
            query.append(q)
            args += a

        supp_query, supp_args = self.__create_supplementary_data_queries(supp_options, supp_ids)

        if supp_query != '':
            query.append(supp_query)
            args += supp_args

        query = '\nINTERSECT\n'.join(query)
        return query, args

    def create_sort_query(self, filter_query, filter_args, experiment_id, sort_by_diff, sort_by_zscore):
        """Creates a sort query.

        Parameters:

        filter_query = filter query from return value of create_filter_query()
        filter_args = filter query arguments from return value of create_filter_query()
        experiment_id = the genes in the query result are sorted by the
        log_likelihood of the given experiment id


        Returns: (query string, query arguments)

        The query string is used to find all filtered genes sorted by
        the log_likelihood of the given experiment result.
        """
        fq = ''
        fa = []
        if filter_query != '':
            fq = """ AND r.gene_id IN (%s)""" % filter_query
            fa = filter_args

        if sort_by_zscore:
            query = """SELECT r.gene_id
                       FROM results AS r, z_scores AS z
                       WHERE r.gene_id = z.gene_id
                         AND r.experiment_id = ? """ + fq + " ORDER BY z.mean_z_score DESC"
        else:
            if sort_by_diff:
                order_by = " ORDER BY (r.log_likelihood - r.baseline_log_likelihood)"
            else:
                order_by = " ORDER BY r.log_likelihood"

            query = """SELECT r.gene_id
                       FROM results AS r
                       WHERE r.experiment_id = ? """ + fq + order_by + " DESC"

        args = [experiment_id] + fa
        return query, args

    def get_gene_ids_for_results(self, sort_query, sort_args):
        """Executes the given query with the given arguments.
        The query and arguments are return values of create_sort_query()

        Returns: [gene id]
        """
        self.c.execute(sort_query, tuple(sort_args))
        return [id for (id,) in self.c.fetchall()]

    def get_results_for_gene_ids(self, experiment_ids, gene_ids):
        """Gets results of the given experiments for the given genes.

        Returns: [(probe name, experiment description, log likelihood, baseline log likelihood, gene id, params, experiment_id)]
        """
        query = """SELECT g.probe_name, e.description, r.log_likelihood, r.baseline_log_likelihood, r.gene_id, r.params, e.experiment_id
                   FROM genes AS g, results AS r, experiment_annotation AS e
                   WHERE g.gene_id = r.gene_id
                     AND r.experiment_id = e.experiment_id
                     AND r.experiment_id IN (%s)
                     AND g.gene_id IN (%s)""" % (self.__to_placeholders(experiment_ids), self.__to_placeholders(gene_ids))
        args = experiment_ids + gene_ids
        self.c.execute(query, tuple(args))
        return self.c.fetchall()

    def get_gene_probe_names_dict(self, gene_ids):
        """Returns a dictionary mappings from gene id to gene probe name.

        Example: {8194: u'FBgn0028420', 6150: u'FBgn0035308', 2577: u'FBgn0011828', 18: u'FBgn0010434'}
        """
        d = {}
        self.c.execute("""SELECT g.gene_id, g.probe_name
                          FROM genes AS g
                          WHERE g.gene_id IN (%s)""" % self.__to_placeholders(gene_ids), gene_ids)
        for (id, name) in self.c.fetchall():
            d[id] = name
        return d

    def get_gene_ids_dict(self, probe_names):
        """Returns a dictionary mapping from gene id to gene probe name.

        Example: {u'FBgn0028420': 8194, u'FBgn0035308': 6150, u'FBgn0011828': 2577, u'FBgn0010434': 18}
        """
        d = {}
        self.c.execute("""SELECT g.gene_id, g.probe_name
                          FROM genes AS g
                          WHERE g.probe_name IN (%s)""" % self.__to_placeholders(probe_names), probe_names)
        for (id, name) in self.c.fetchall():
            d[name] = id
        return d

    def get_highlights(self, supp_options, supp_ids, gene_ids):
        """Gets highlighted gene ids for the given supplementary options.

        Returns a dictionary mapping from supplementary dataset name to a list of gene ids
        Example:
        {u'ischip': [3341], u'chipdist': [3341, 8354], u'hasinsitu': [3341]}
        """
        if not gene_ids:
            return {}
        highlights = {}
        for (name, symbol, value) in supp_options:
            q, a = self.__create_supplementary_data_query(supp_ids[name], symbol, float(value), gene_ids)
            self.c.execute(q, a)
            highlights[name] = [id for (id,) in self.c.fetchall()]
        return highlights

    def get_experiment_parameter_count(self, experiment_id):
        """Gets the number_of_parameters field for the given experiment.
        """
        query = """SELECT e.number_of_parameters
                   FROM experiment_annotation AS e
                   WHERE e.experiment_id = ?"""
        self.c.execute(query, (experiment_id,))
        result = self.c.fetchone()
        if result:
            return result[0]

    def get_experiment_parameter_names(self, experiment_id):
        """Gets the parameters_names field for the given experiment.
        """
        query = """SELECT e.parameter_names
                   FROM experiment_annotation AS e
                   WHERE e.experiment_id = ?"""
        self.c.execute(query, (experiment_id,))
        result = self.c.fetchone()
        if result:
            return result[0]

    def get_experiment_ids_names(self, experiment_ids):
        """Returns a list of experiment_ids and the corresponding names.
        Example:
            [(1, "GPSIM"), (2, "GPDISIM"), (3, "GPSIM"), (4, "GPDISIM"), (5, "MarLL")]
        """
        query = """SELECT e.experiment_id, e.name
                   FROM experiment_annotation AS e
                   WHERE e.experiment_id IN (%s)""" % self.__to_placeholders(experiment_ids)
        self.c.execute(query, experiment_ids)
        return self.c.fetchall()

    def get_figuredata(self, figure_id, gene_id):
        """Returns figure blob data corresponding the given arguments.

        Returns: binary blob
        """
        query = """SELECT f.data
                   FROM figuredata AS f
                   WHERE f.figure_id = ?
                     AND f.gene_id = ?"""
        self.c.execute(query, (figure_id, gene_id))
        result = self.c.fetchone()
        if result:
            return result[0]

    def get_version(self):
        """Returns the database version (user_version in SQLite).
        """
        self.c.execute("PRAGMA user_version")
        return self.c.fetchone()[0]

    def get_gene_count(self):
        """Returns the number of genes in the database.
        """
        self.c.execute("SELECT COUNT(*) FROM genes")
        return self.c.fetchone()[0]




try:
    import urllib.request, urllib.parse, urllib.error # Python >=3.0
except ImportError:
    import urllib # Python <3.0

def print_form_javascript(filter_count, filters_data):
    """Prints out the javascript code that makes adding and removing filters
    dynamically work. It works by using jQuery Dynamic Form plugin
    (http://code.google.com/p/jquery-dynamic-form/).
    The filters are parsed from the CGI query and then to this function
    in JSON format as 'filters_data' parameter.

    Parameters:
        filter_count: maximum number of filters

        filters_data: existing filters in JSON format
        The format is as follows:
        [{"filterby":<FILTER_VARIABLE>, "filtersymbol":<FILTER_SYMBOL>, "threshold":<FILTER_THRESHOLD>}]

        Example:
        [{"filterby":"GPDISIM", "filtersymbol":">", "threshold":"10"},
         {"filterby":"GPDISIM_diff", "filtersymbol":">", "threshold":"0"},
         {"filterby":"zscore", "filtersymbol":">", "threshold":"1.8"},
         {"filterby":"zscore", "filtersymbol":"<", "threshold":"5"},]
    """
    print("""<script type="text/javascript">
    $(document).ready(function() {
        $("#filter").dynamicForm("#add_filter", "#remove_filter",
        {
            limit:%d,
            normalizeFullForm:false,
            data:%s
        });
    });
</script>""" % (filter_count, filters_data))

def print_headers(title, filter_count=20, filters_data=""):
    """Prints the headers of the html document (<head> and the beginning
    of <body>). Prints the page header.

    Parameters:
        title: page title and header
        filter_count & filters_data: see function print_form_javascript()
    """
    print("Content-Type: text/html")     # HTML is following
    print("")                            # blank line, end of headers
    print("""<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
<html>
<head>
  <title>%s</title>
  <link rel="stylesheet" type="text/css" href="tigreBrowser.css"/>
  <script type="text/javascript" src="jquery-1.4.2.min.js"></script>
  <script type="text/javascript" src="jquery-dynamic-form.js"></script>
  <script type="text/javascript" src="tigreBrowser.js"></script>
  """ % title)
    print_form_javascript(filter_count, filters_data)
    print("""</head>
<body>
<a href="tigreBrowser.cgi"><h1>%s</h1></a>""" % title)

def print_footer():
    """Prints the footer of the html document (</body> and </html>)
    """
    print("""</body>
</html>""")

def print_select(formdata, key, values, default=None):
    print("""<SELECT name="%s" id="%s">""" % (key, key))
    if key in formdata:
        for k in values:
            if formdata.getvalue(key) == k:
                print("<OPTION selected>%s</OPTION>" % k)
            else:
                print("<OPTION>%s</OPTION>" % k)
    else:
        for k in values:
            if default == k:
                print("<OPTION selected>%s</OPTION>" % k)
            else:
                print("<OPTION>%s</OPTION>" % k)
    print("""</SELECT>""")

def print_dummy_select(formdata, key, values, default=None):
    selected = values[0]
    if key in formdata:
        for k in values:
            if formdata.getvalue(key) == k:
                selected = k
    else:
        if default is not None:
            selected = default
    print("""<INPUT type="hidden" name="%s" value="%s">""" % (key, selected))

def print_checkbox(formdata, key, clicked=False):
    if key in formdata or clicked:
        print("""<INPUT type="checkbox" name="%s" id="%s" checked value="yes"/>""" % (key, key))
    else:
        print("""<INPUT type="checkbox" name="%s" id="%s" value="yes"/>""" % (key, key))

def print_radiobutton(formdata, key, value, default=False):
    if key in formdata and formdata.getvalue(key) == value or (default and not formdata.getvalue(key)):
        print("""<INPUT type="radio" name="%s" checked value="%s"/>""" % (key, value))
    else:
        print("""<INPUT type="radio" name="%s" value="%s"/>""" % (key, value))

def print_input(formdata, key, default, size="4"):
    if key in formdata:
        print("""<INPUT type="text" name="%s" value="%s" size="%s"/>""" % (key, formdata.getvalue(key), str(size)))
    else:
        print("""<INPUT type="text" name="%s" value="%s" size="%s"/>""" % (key, default, str(size)))

def print_genes(genes, aliases=None):
    """Prints gene names as tables.
    """
    print("""<table class="stats"><tr><td>""")
    if aliases is not None:
        for alias in aliases:
            print(alias + "<br/>")
        print("</td><td>")
    for gene in genes:
        print(gene + "<br/>")
    print("</td><td>")
    print("{")
    genes = ["'" + gene + "'" for gene in genes]
    genes = ',<br/>'.join(genes)
    print(genes)
    print("};")
    print("</td></tr></table>")

def print_pages(number, active, req, script, pagesize):
    if number < 1:
        return
    req.pop('offset', None)
    script = script or ""
    display_count = 10

    try:
        url = script + '?' + urllib.parse.urlencode(req, True) # Python >=3.0
    except AttributeError:
        url = script + '?' + urllib.urlencode(req, True) # Python <3.0

    def print_link(offset, name):
        print("""<a href="%s">%s</a>""" % (url + '&offset=' + str(offset), name))

    print("<p>Pages: ")

    # Previous
    if active > 0:
        print_link((active - 1) * pagesize, "Previous")
    elif number > 1:
        print("<strong>Previous</strong> ")

    if active > display_count - 1:
        for k in range(0, display_count // 2):
            print_link(k * pagesize, k + 1)
        min = display_count // 2
        max = active - display_count + 2
        print_link(int(pagesize * (min + max) // 2), "...")

    for k in range(number):
        if abs(active - k) < display_count:
            if k == active:
                print("<strong>%d</strong> " % (k+1))
            else:
                print_link(k * pagesize, k + 1)

    if active < number - display_count:
        min = active + display_count
        max = number - display_count // 2 + 1
        print_link(int(pagesize * (min + max) // 2), "...")
        for k in range(number - display_count // 2, number):
            print_link(k * pagesize, k + 1)

    # Next
    if active < (number - 1):
        print_link((active + 1) * pagesize, "Next")
    elif number > 1:
        print("<strong>Next</strong>")

    print("</p>")

def print_dict_table(dict):
    keys = list(dict.keys())
    keys.sort()
    print("""<table class="stats">""")
    for k in keys:
        print("<tr>")
        value = dict[k]
        if isinstance(value, list):
            value = ', '.join(value)
        print("<td>%s:</td><td>%.2f</td>" % (k.replace("_", ' '), value))
        print("</tr>")
    print("</table>")

def print_alias_dict(dict):
    keys = list(dict.keys())
    keys.sort()
    print("""<table class="stats">""")
    for alias_class in keys:
        print("""<tr class="alias_annotation_%s">""" % alias_class)
        alias = dict[alias_class]
        if alias == [None]:
            alias = 'N/A'
        if isinstance(alias, list):
            alias = ', '.join(alias)
        print("<td>%s</td><td>%s</td>" % (alias_class, alias))
        print("</tr>")
    print("</table>")

def print_figure_cell(figure, alt='', class_attr=''):
    print("""<td class="%s">""" % class_attr)
    print("<a href=%s>" % figure)
    print("""<img src="%s" width=400 height=300 alt="%s">""" % (figure, alt))
    print("</a>")
    print("</td>")

def print_highlights_table(colors, centered=False):
    if not colors:
        return
    margin = centered and "auto" or "none"
    print("""<table class="color_table" style="margin-left:%s; margin-right:auto">""" % margin)
    print("<tr>")
    for color in colors:
        print("""<td class="color" width=10 height=10 bgcolor="#%s"></td>""" % (color))
    print("</tr>")
    print("</table>")





#! /usr/bin/env python
import sys
from optparse import OptionParser, OptionGroup
from tigreBrowser.database_import import *

def main():
    usage = "usage: %prog [options] database_file alias_class dataset_name"
    parser = OptionParser(usage)
    group_alias = OptionGroup(parser, "Alias annotation")
    group_data = OptionGroup(parser, "Expression dataset annotation")

    parser.add_option('-f', '--file', dest='filename',
            help='read data from FILENAME instead of stdin')
    parser.add_option('-c', '--column', dest='column', type='int',
            action='store', help='data column [default: %default]', default=2)
    parser.add_option('-d', '--delimiter', dest='delimiter', type='string',
            action='store', help='column delimiter [default: \'%default\']', default=None)

    group_alias.add_option('--alias-source', dest='alias_source', type='string',
            action='store', help='set alias source')
    group_alias.add_option('--alias-desc', dest='alias_desc', type='string',
            action='store', help='set alias description')

    group_data.add_option('--dataset-desc', dest='dataset_desc', type='string',
            action='store', help='set dataset description')
    group_data.add_option('--species', dest='dataset_species', type='string',
            action='store', help='set dataset species')
    group_data.add_option('--source', dest='dataset_source', type='string',
            action='store', help='set dataset source')
    group_data.add_option('--platform', dest='dataset_platform', type='string',
            action='store', help='set dataset platform')
    group_data.add_option('--save-location', dest='dataset_save_location', type='string',
            action='store', help='set dataset save location')
    group_data.add_option('--figure-filename', dest='dataset_figure_filename', type='string',
            action='store', help='set dataset figure filename')

    parser.add_option_group(group_alias)
    parser.add_option_group(group_data)

    (options, args) = parser.parse_args()

    if len(args) < 3:
        print('Not enough arguments. Use -h to show help.')
        sys.exit()

    database_file = args[0]
    alias_class = args[1]
    dataset_name = args[2]
    annotation = [alias_class, options.alias_source, options.alias_desc]
    dataset_annotation = [dataset_name, options.dataset_species, options.dataset_source, options.dataset_platform, options.dataset_desc, options.dataset_save_location, options.dataset_figure_filename]

    db = gpsim_database(database_file)

    try:
        print('Trying to create tables...')
        db.create_tables()
        print('Tables created')
    except sqlite3.OperationalError:
        print('Tables already exist')

    probe_names, data = get_data(options.filename, options.column - 1, options.delimiter)
    db.add_aliases(probe_names, data, annotation, dataset_annotation)

def get_data(file, column, delim):
    probe_names = []
    data = []

    def add_data(line):
        line = line.strip().split(delim)
        for alias in line[column:]:
            probe_names.append(line[0].strip())
            data.append(alias)

    if not file:
        for line in sys.stdin.readlines():
            add_data(line)
    else:
        for line in open(file):
            add_data(line)
    return probe_names, data


if __name__ == "__main__":
    main()



#! /usr/bin/env python
import sys
from optparse import OptionParser, OptionGroup
from tigreBrowser.database_import import *
from datetime import date

def main():
    usage = "usage: %prog [options] database_file experiment_name regulator_name dataset_name"
    parser = OptionParser(usage)
    group_exp = OptionGroup(parser, "Experiment annotation")
    group_data = OptionGroup(parser, "Expression dataset annotation")

    parser.add_option('-f', '--file', dest='filename',
            help='read data from FILENAME instead of stdin')
    parser.add_option('-d', '--delimiter', dest='delimiter', type='string',
            action='store', help='column delimiter [default: whitespace]', default=None)

    group_exp.add_option('-m', '--model-translation', dest='model_translation',
            action='store_true', help='set model translation to TRUE [default: False]', default=False)
    group_exp.add_option('-l', '--loop-variable', dest='loop_variable', type='int',
            action='store', help='set loop variable [default: %default]', default=2)
    group_exp.add_option('--number-of-params', dest='number_of_params', type='int',
            action='store', help='set the number of experiment parameters')
    group_exp.add_option('--parameter-names', dest='parameter_names', type='string',
            action='store', help='set the experiment parameter names')
    group_exp.add_option('-p', '--producer', dest='producer', type='string',
            action='store', help='set the experiment producer')
    group_exp.add_option('-t', '--timestamp', dest='timestamp', type='string',
            action='store', help='set the experiment timestamp YYYY-MM-DD [default: current date]', default=date.today().isoformat())
    group_exp.add_option('--experiment-desc', dest='experiment_desc', type='string',
            action='store', help='set experiment description')
    group_exp.add_option('--experiment-set', dest='experiment_set', type='string',
            action='store', help='add results to this experiment set')

    group_data.add_option('--dataset-desc', dest='dataset_desc', type='string',
            action='store', help='set dataset description')
    group_data.add_option('--species', dest='dataset_species', type='string',
            action='store', help='set dataset species')
    group_data.add_option('--source', dest='dataset_source', type='string',
            action='store', help='set dataset source')
    group_data.add_option('--platform', dest='dataset_platform', type='string',
            action='store', help='set dataset platform')
    group_data.add_option('--save-location', dest='dataset_save_location', type='string',
            action='store', help='set dataset save location')
    group_data.add_option('--figure-filename', dest='dataset_figure_filename', type='string',
            action='store', help='set dataset figure filename')

    parser.add_option_group(group_exp)
    parser.add_option_group(group_data)

    (options, args) = parser.parse_args()

    if len(args) < 3:
        print('Not enough arguments. Use -h to show help.')
        sys.exit()

    database_file = args[0]
    experiment_name = args[1]
    regulator_name = args[2]
    dataset_name = args[3]
    
    experiment_annotation = [regulator_name, options.loop_variable, options.model_translation, options.number_of_params, options.parameter_names, options.producer, options.timestamp, options.experiment_desc, experiment_name]
    dataset_annotation = [dataset_name, options.dataset_species, options.dataset_source, options.dataset_platform, options.dataset_desc, options.dataset_save_location, options.dataset_figure_filename]

    db = gpsim_database(database_file)

    try:
        print('Trying to create tables...')
        db.create_tables()
        print('Tables created')
    except sqlite3.OperationalError:
        print('Tables already exist')

    figure_annotations = get_figures(options.filename, options.delimiter)
    for annotation in figure_annotations:
        db.add_figure(experiment_annotation, dataset_annotation, annotation)

def get_figures(file, delim):
    figures = []
    
    def add_figure(line):
        line = line.strip().split(delim)
        annotation = [line[0].strip(), None, None, None]
        try:
            annotation[1] = line[1].strip()
            annotation[2] = line[2].strip()
            annotation[3] = int(line[3])
        except:
            pass
        figures.append(annotation)

    if not file:
        for line in sys.stdin.readlines():
            add_figure(line)
    else:
        for line in open(file):
            add_figure(line)
    return figures

if __name__ == "__main__":
    main()



#! /usr/bin/env python
import sys
from optparse import OptionParser, OptionGroup
from tigreBrowser.database_import import *
from datetime import date

ROOT_EXPERIMENT_SET = 'All experiments'

LOOP_TFS = 1

def is_header_line(line, columns):
    try:
        for col in columns:
            if col != None:
                float(line[col])
        return False
    except ValueError:
        return True

def main():
    usage = "usage: %prog [options] database_file dataset_name experiment_name regulator_name experiment_figure"
    parser = OptionParser(usage)
    group_exp = OptionGroup(parser, "Experiment annotation")
    group_data = OptionGroup(parser, "Expression dataset annotation")
    group_figure = OptionGroup(parser, "Figure annotation")

    parser.add_option('-f', '--file', dest='filename',
            help='read data from FILENAME instead of stdin')
    parser.add_option('--log-likelihood-column', dest='log_column', type='int',
            action='store', help='log likelihood column in results [default: %default]', default=2)
    parser.add_option('--baseline-log-likelihood-column', dest='baseline_column', type='int',
            action='store', help='baseline log likelihood column in results (0=no results) [default: %default]', default=0)
    parser.add_option('-d', '--delimiter', dest='delimiter', type='string',
            action='store', help='column delimiter [default: whitespace]', default=None)

    group_exp.add_option('-m', '--model-translation', dest='model_translation',
            action='store_true', help='set model translation to TRUE [default: False]', default=False)
    group_exp.add_option('-l', '--loop-variable', dest='loop_variable', type='int',
            action='store', help='set loop variable [default: %default]', default=2)
    group_exp.add_option('--targets-file', dest='targets_file', type='string',
            action='store', help='set filename of target genes, this must be set if loop variable is se to %d' % LOOP_TFS, default=None)
    group_exp.add_option('--number-of-params', dest='number_of_params', type='int',
            action='store', help='set the number of experiment parameters')
    group_exp.add_option('--parameter-names', dest='parameter_names', type='string',
            action='store', help='set the experiment parameter names')
    group_exp.add_option('-p', '--producer', dest='producer', type='string',
            action='store', help='set the experiment producer')
    group_exp.add_option('-t', '--timestamp', dest='timestamp', type='string',
            action='store', help='set the experiment timestamp YYYY-MM-DD [default: current date]', default=date.today().isoformat())
    group_exp.add_option('--experiment-desc', dest='experiment_desc', type='string',
            action='store', help='set experiment description')
    group_exp.add_option('--experiment-set', dest='experiment_set', type='string',
            action='store', help='add results to this experiment set')

    group_data.add_option('--dataset-desc', dest='dataset_desc', type='string',
            action='store', help='set dataset description')
    group_data.add_option('--species', dest='dataset_species', type='string',
            action='store', help='set dataset species')
    group_data.add_option('--source', dest='dataset_source', type='string',
            action='store', help='set dataset source')
    group_data.add_option('--platform', dest='dataset_platform', type='string',
            action='store', help='set dataset platform')
    group_data.add_option('--save-location', dest='dataset_save_location', type='string',
            action='store', help='set dataset save location')
    group_data.add_option('--figure-filename', dest='dataset_figure_filename', type='string',
            action='store', help='set dataset figure filename')

    group_figure.add_option('--figure-name', dest='figure_name', type='string',
            action='store', help='set experiment figure name')
    group_figure.add_option('--figure-desc', dest='figure_desc', type='string',
            action='store', help='set experiment figure description')


    parser.add_option_group(group_exp)
    parser.add_option_group(group_data)
    parser.add_option_group(group_figure)

    (options, args) = parser.parse_args()

    if len(args) < 4:
        print('Not enough arguments. Use -h to show help.')
        sys.exit()

    database_file = args[0]
    dataset_name = args[1]
    experiment_name = args[2]
    regulator_name = args[3] if options.loop_variable != LOOP_TFS else None
    figure_filename = args[4] if len(args) > 4 else None

    dataset_annotation = [dataset_name, options.dataset_species, options.dataset_source, options.dataset_platform, options.dataset_desc, options.dataset_save_location, options.dataset_figure_filename]
    if not options.experiment_desc:
        options.experiment_desc = experiment_name
    experiment_annotation = [regulator_name, options.loop_variable, options.model_translation, options.number_of_params, options.parameter_names, options.producer, options.timestamp, options.experiment_desc, experiment_name]

    figure_annotation = [figure_filename, options.figure_name, options.figure_desc, 0]

    targets_file = options.targets_file
    if not targets_file and options.loop_variable == LOOP_TFS:
        print('Targets filename must be set when loop variable is 1')

    db = gpsim_database(database_file)

    try:
        print('Trying to create tables...')
        db.create_tables()
        print('Tables created')
    except sqlite3.OperationalError:
        print('Tables already exist')

    baseline_col = options.baseline_column - 1 if options.baseline_column != 0 else None

    probe_names, log_likelihoods, baseline_log_likelihoods = get_results(options.filename, options.log_column - 1, baseline_col, options.delimiter)

    params = [None] * len(probe_names)
    db.add_experiment_results(probe_names, log_likelihoods, baseline_log_likelihoods, params, experiment_annotation, dataset_annotation)

    if figure_filename:
        db.add_figure(experiment_annotation, dataset_annotation, figure_annotation)

    if options.experiment_set:
        db.add_experiment_set(options.experiment_set, ROOT_EXPERIMENT_SET, [experiment_annotation], [dataset_annotation])
    else: # put all experiments under ROOT_EXPERIMENT_SET
        db.add_experiment_set(ROOT_EXPERIMENT_SET, None, [experiment_annotation], [dataset_annotation])


    if targets_file:
        targets = []
        for line in open(targets_file):
            targets.append(line.strip())
        db.add_target_sets(targets, experiment_annotation, dataset_annotation)


def get_results(file, log_column, baseline_column, delimiter):
    probe_names = []
    log_likelihoods = []
    baseline_log_likelihoods = []

    def add_results(line):
        line = line.strip().split(delimiter)
        if is_header_line(line, [log_column, baseline_column]):
            return

        probe_names.append(line[0].strip())
        log_likelihoods.append(float(line[log_column]))
        if baseline_column:
            baseline_log_likelihoods.append(float(line[baseline_column]))
        else:
            baseline_log_likelihoods.append(None)

    if not file:
        for line in sys.stdin.readlines():
            add_results(line)
    else:
        for line in open(file):
            add_results(line)
    return probe_names, log_likelihoods, baseline_log_likelihoods


if __name__ == "__main__":
    main()




#!/usr/bin/env python

import os
from setuptools import setup
from glob import glob

README = open(os.path.join(os.path.dirname(__file__), 'README.md')).read()

setup(name='tigreBrowser',
      version='1.1',
      license='AGPL3',
      author='Miika-Petteri Matikainen',
      maintainer='Antti Honkela',
      maintainer_email='antti.honkela@helsinki.fi',
      url='https://github.com/PROBIC/tigreBrowser/',
      description='Gene expression model browser for results from tigre R package (http://www.bioconductor.org/packages/release/bioc/html/tigre.html)',
      long_description=README,
      scripts=['scripts/tigreServer.py', 'scripts/insert_aliases.py',
               'scripts/insert_figures.py', 'scripts/insert_results.py',
               'scripts/insert_supplementary_data.py',
               'scripts/insert_zscores.py'],
      packages=['tigreBrowser'],
      package_data={'tigreBrowser': ['cgi/tigreBrowser/*']},
      data_files=[('share/tigreBrowser', ['database.sqlite'])],
      install_requires=['setuptools'],
      classifiers=[
          'Environment :: Web Environment',
          'License :: OSI Approved :: GNU Affero General Public License v3',
          'Natural Language :: English',
          'Operating System :: OS Independent',
          'Programming Language :: Python',
          'Topic :: Database :: Front-Ends',
          'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
          'Topic :: Scientific/Engineering :: Bio-Informatics',
      ],
      )



from tigreBrowser.database import *
import xdrlib

class Results:
    """This class is responsible for getting the results from the database
    using functions in database.py.
    """
    def __init__(self, db, experiment_set, regulator, target_set, experiment, filters, supplementary_options, search_genes):
        """Initialization. Does not, however, fetch the results.
        Parameters:
            db: database object

            experiment_set: name of the experiment set

            regulator: name of the regulator if it exists

            target_set: name of the target set if it exists

            experiment: name of the experiment, appended with '_diff'
                        if sorting is to be done by diff.
                        Can also be 'zscore' if sorting is done by z-scores

            filters: filters that will be applied when fetching
                     and filtering results (see create_filter_query() in
                     database.py for explanation about the parameter type)

            supplementary_options: highlights will be fetched according
                                   to these options (see create_filter_query()
                                   in database.py for explanation about
                                   the parameter type)

            search_genes: search these gene names or aliases (list of strings)
        """
        self.__db = db
        self.__set_id = db.get_experiment_set_id(experiment_set)
        self.__reg_id = db.get_regulator_id(regulator)
        self.__set_experiment_ids = db.get_experiment_ids_in_set_recursively(self.__set_id)

        if target_set:
            self.__target_ids = db.get_gene_ids_dict(target_set).values()
        self.__experiment_id = db.get_experiment_id(experiment.replace('_diff', ''), self.__reg_id)
        self.__sort_by_diff = experiment.endswith('_diff')
        self.__sort_by_zscore = False

        if experiment == 'zscore':
            self.__experiment_id = self.__set_experiment_ids[0]
            self.__sort_by_zscore = True

        if not self.__experiment_id:
            raise Exception('No experiment with the current TF')

        self.__dataset_id = db.get_experiment_dataset_id(self.__experiment_id)
        self.__supplementary_annotation_ids_dict = self.__create_supplementary_annotation_ids_dict(self.__reg_id)
        self.__filters = filters
        self.__supplementary_options = supplementary_options

        regulator_experiment_ids = db.get_experiment_ids(self.__reg_id)

        # If a regulator is defined (TARGET_RANKING),
        # choose only the experiments in a set that have the given regulator.
        # Use set intersection.
        if self.__reg_id:
            self.__set_experiment_ids = list(set(self.__set_experiment_ids) & set(regulator_experiment_ids))

        self.__search_gene_ids = None
        if search_genes:
            self.__search_gene_ids = db.get_gene_ids_by_name_in_experiments(self.__set_experiment_ids, search_genes)

        if not self.__set_experiment_ids:
            raise Exception('No results for the given selection')

        for (name, symbol, value) in supplementary_options:
            if name not in self.__supplementary_annotation_ids_dict:
                raise Exception('Supplementary option %s not available for the given TF' % name)

        self.__results_all = {}
        self.__aliases_all = {}
        self.__params_all = {}
        self.__supps_all = {}
        self.__zscores_all = {}
        self.__probe_names = {}
        self.__highlights = {}

    def __create_supplementary_annotation_ids_dict(self, reg_id):
        d = {}
        for (name, ann_id) in self.__db.get_supplementary_annotation_ids(reg_id):
            d[name] = ann_id
        return d

    def __parse_results(self, results):
        self.__experiment_parameter_names = self.__get_experiment_parameter_names()
        result_gene_ids = set()
        probe_names = {}
        results_all = {}
        params_all = {}
        for row in results:
            probe_name = row[0]
            desc = row[1]
            likelihood = row[2]
            baseline_likelihood = row[3]
            gene_id = row[4]
            param_values = row[5]
            exp_id = row[6]
            result_gene_ids.add(gene_id)

            results_all.setdefault(gene_id, {})[desc] = likelihood

            # add results with appended '_baseline' and '_diff'
            # if baseline is defined to show them in results listing
            if baseline_likelihood:
                results_all[gene_id][desc + '_baseline'] = baseline_likelihood
                results_all[gene_id][desc + '_diff'] = likelihood - baseline_likelihood
            probe_names[gene_id] = probe_name
            params_all.setdefault(gene_id, {})[exp_id] = self.__map_params_to_names(exp_id, self.__experiment_parameter_names.get(exp_id, []), param_values)
        return result_gene_ids, probe_names, results_all, params_all

    def __query_supplementary_datas(self, gene_ids, supp_ids):
        supps = self.__db.get_gene_supplementary_datas(gene_ids, supp_ids)
        d = {}
        for (supp_id, name, value) in supps:
            d.setdefault(supp_id, {})[name] = value
        return d

    def __query_gene_aliases(self, gene_ids):
        aliases = self.__db.get_gene_aliases(gene_ids)
        d = {}
        for (alias_id, alias_class, alias) in aliases:
            d.setdefault(alias_id, {}).setdefault(alias_class, []).append(alias)
        return d

    def __query_zscores(self, gene_ids, dataset_id):
        d = {}
        for (gene_id, zscore) in self.__db.get_z_scores(gene_ids, dataset_id):
            d[gene_id] = zscore
        return d

    def __parse_rdata_double_raw_vector(self, data_buffer):
        # Python 2.5 and 3.x compatibility code
        try:
            header = [ord(x) for x in [data_buffer[0], data_buffer[1]]]
        except TypeError:
            header = [data_buffer[0], data_buffer[1]]

        # RData binary format
        if header[0] != ord('X') or header[1] != ord('\n'):
            return None
        xdr = xdrlib.Unpacker(data_buffer[2:])
        xdr.unpack_int()
        xdr.unpack_int()
        xdr.unpack_int()
        xdr.unpack_int()
        xdr.unpack_int()
        data = []
        while True:
            try:
                data.append(xdr.unpack_double())
            except EOFError:
                break
        return data

    def __map_params_to_names(self, exp_id, param_names, param_values):
        if not param_values:
            return {}
        param_values = self.__parse_rdata_double_raw_vector(param_values)
        return dict(zip(param_names, param_values))

    def fetch_results(self, number_of_genes, offset):
        """Fetches the results from the database.

        Parameters:
            number_of_genes: fetch at most this many genes

            offset: offset in the results listing

        Returns: (gene ids for the result genes, total number of genes with results)
        """
        fq, fa = self.__db.create_filter_query(self.__reg_id, self.__filters, [], self.__supplementary_annotation_ids_dict)
        sq, sa = self.__db.create_sort_query(fq, fa, self.__experiment_id, self.__sort_by_diff, self.__sort_by_zscore)

        if self.__search_gene_ids != None:
            gene_ids = self.__search_gene_ids
        else:
            gene_ids = self.__db.get_gene_ids_for_results(sq, sa)

        count = len(gene_ids) # total result count
        gene_ids = gene_ids[offset:(offset + number_of_genes)] # OFFSET, LIMIT

        results = self.__db.get_results_for_gene_ids(self.__set_experiment_ids, gene_ids)
        result_gene_ids, self.__probe_names, self.__results_all, self.__params_all = self.__parse_results(results)

        if not results: # quick fix for the result count
            count = 0

        # remove gene ids that are in gene_ids but not in result_gene_ids
        l = gene_ids[:]
        for gene_id in l:
            if gene_id not in result_gene_ids:
                gene_ids.remove(gene_id)

        self.__supps_all = self.__query_supplementary_datas(gene_ids, self.__supplementary_annotation_ids_dict)
        self.__aliases_all = self.__query_gene_aliases(gene_ids)
        self.__zscores_all = self.__query_zscores(gene_ids, self.__dataset_id)

        self.__highlights = self.__db.get_highlights(self.__supplementary_options, self.__supplementary_annotation_ids_dict, gene_ids)

        return gene_ids, count

    def get_experiment_results(self, gene_id):
        """Returns a dictionary of experiment results for the given gene_id.

        Returns: {experiment_name, log_likelihood or baseline_log_likelihood}

        Example:
            {'GPDISIM_diff': 494.65294095332217, 'GPDISIM': 6.7597099032014309, 'GPDISIM_baseline': -487.89323105012073}
        """
        return self.__results_all.get(gene_id)

    def get_zscore(self, gene_id):
        """Returns z-score for the given gene_id.
        """
        return self.__zscores_all.get(gene_id)

    def get_supplementary_data(self, gene_id):
        """Returns a dictionary representing supplementary data
        for the given gene_id.

        Returns: {supplementary dataset name, value}

        Example:
            {'ischip': 1.0, 'chipdist': 1671.0, 'isinsitu': 0.0, 'hasinsitu': 0.0}
        """
        return self.__supps_all.get(gene_id, {})

    def get_parameters(self, gene_id):
        """Returns a dictionary of parameters in different experiments
        for the given gene id.

        Returns: {experiment id: {parameter name: parameter value}}

        Example:
            {1: {'rbf1_variance/disim1_rbf_variance': 0.59039292169555113,
                 'disim1_variance': -5.9057868788275316,
                 'disim1_decay': -3.9377851775471258,
                 'disim1_di_variance': 0.0,
                 'Basal1': -8.9106876980653453,
                 'disim1_di_decay': -4.0190835928767878,
                 'rbf1_inverseWidth/disim1_inverseWidth': -0.51096542027712455}
            }
        """
        return self.__params_all.get(gene_id, {})

    def get_aliases(self, gene_id):
        """Returns a dictionary of aliases for the given gene id.

        Returns: {alias class, [aliases]}

        Example:
            {'ENTREZID': [u'38211'],
             'SYMBOL': [u'CG12011'],
             'FLYBASE': [u'FBgn0035257'],
             'GENENAME': [u'CG12011 gene product from transcript CG12011-RA']}
        """
        return self.__aliases_all.get(gene_id, {})

    def get_probe_name(self, gene_id):
        """Returns the probe name for the given gene id.
        """
        return self.__probe_names.get(gene_id)

    def get_highlights(self):
        """Gets a dictionary of supplementary dataset names to gene ids that
        will be highlighted.

        Returns: {supplementary dataset name: [gene ids]}

        Example:
            {'ischip': [4632, 8354, 10609], 'isinsitu': [], 'hasinsitu': [4632, 8354, 10609]}
        """
        return self.__highlights

    def get_dataset_figure(self):
        """Gets the template URL to the dataset figure.

        Example:
            http://www.something.com/something/figures/${probe_name}.png
        """
        return self.__db.get_dataset_figure_filename(self.__dataset_id)

    def get_experiment_figures(self):
        """Gets experiment figure annotations.

        Returns: [(figure id, filename, name, description, priority)]
        """
        return self.__db.get_experiment_figures(self.__set_experiment_ids)

    def get_alias_annotations(self):
        """Gets all alias annotations.

        Returns: [(alias annotation id, alias class, source, description)]
        """
        return self.__db.get_alias_annotations(self.__dataset_id)

    def get_experiment_names(self):
        """Gets a dictionary mapping from experiment ids to corresponding names.

        Returns: {experiment id: experiment name}

        Example:
            {1: 'GPDISIM', 2: 'GPSIM'}
        """
        exp_dict = {}
        results = self.__db.get_experiment_ids_names(self.__set_experiment_ids)
        for (exp_id, name) in results:
            exp_dict[exp_id] = name
        return exp_dict

    def __get_experiment_parameter_names(self):
        params_dict = {}
        for exp_id in self.__set_experiment_ids:
            names = self.__db.get_experiment_parameter_names(exp_id)
            if not names:
                continue
            names = [name.strip() for name in names.strip().split(',')]
            params_dict[exp_id] = names
        return params_dict

    def get_experiment_parameter_names(self):
        """Gets a dictionary mapping from experiment ids to a list of
        parameter names.

        Returns: {experiment id: [parameter names]}

        Example:
            {1: ['rbf1_inverseWidth/disim1_inverseWidth',
                 'rbf1_variance/disim1_rbf_variance',
                 'disim1_di_decay',
                 'disim1_di_variance',
                 'disim1_decay',
                 'disim1_variance',
                 'Basal1']
            }
        """
        return self.__experiment_parameter_names

    def get_all_parameter_names(self):
        """Gets a list of parameters names in all experiments.

        Returns: [parameter names]
        """
        all_names = sum(self.__experiment_parameter_names.values(), [])
        # remove duplicates
        names = []
        [names.append(name) for name in all_names if not names.count(name)]
        return names




#! /usr/bin/env python
import sys
from optparse import OptionParser, OptionGroup
from tigreBrowser.database_import import *

def is_header_line(line, columns):
    try:
        for col in columns:
            if col != None:
                float(line[col])
        return False
    except ValueError:
        return True

def main():
    usage = "usage: %prog [options] database_file supplementary_data_name"
    parser = OptionParser(usage)
    group_supp = OptionGroup(parser, "Supplementary dataset annotation")

    parser.add_option('-f', '--file', dest='filename',
            help='read data from FILENAME instead of stdin')
    parser.add_option('-c', '--column', dest='column', type='int',
            action='store', help='data column [default: %default]', default=2)
    parser.add_option('-d', '--delimiter', dest='delimiter', type='string',
            action='store', help='column delimiter [default: whitespace]', default=None)

    group_supp.add_option('-t', '--type', dest='type', type='int',
            action='store', help='set data type (0=boolean, 1=integer, 2=float) [default: 0]', default=0)
    group_supp.add_option('--tf', dest='tf', type='string',
            action='store', help='set TF if this data depends on a TF')
    group_supp.add_option('--source', dest='source', type='string',
            action='store', help='set supplementary data source')
    group_supp.add_option('--plaftorm', dest='platform', type='string',
            action='store', help='set supplementary data platform')
    group_supp.add_option('--desc', dest='desc', type='string',
            action='store', help='set supplementary data desc')

    parser.add_option_group(group_supp)

    (options, args) = parser.parse_args()

    if len(args) < 2:
        print('Not enough arguments. Use -h to show help.')
        sys.exit()

    database_file = args[0]
    name = args[1]
    if not options.desc:
        options.desc = name
    annotation = [options.type, name, options.tf, options.source, options.platform, options.desc]

    db = gpsim_database(database_file)

    try:
        print('Trying to create tables...')
        db.create_tables()
        print('Tables created')
    except sqlite3.OperationalError:
        print('Tables already exist')

    probe_names, data = get_data(options.filename, options.column - 1, options.delimiter)
    db.add_supplementary_data(probe_names, data, annotation)

def get_data(file, column, delim):
    probe_names = []
    data = []

    def add_data(line):
        line = line.strip().split(delim)
        if is_header_line(line, [column]):
            return

        probe_names.append(line[0].strip())
        data.append(float(line[column]))

    if not file:
        for line in sys.stdin.readlines():
            add_data(line)
    else:
        for line in open(file):
            add_data(line)
    return probe_names, data


if __name__ == "__main__":
    main()
